-- EE552 Sign Language Glove Project

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library work;
use work.SLG_pkg.all;
use work.LCD_pkg.all;

entity LCD is
	port(clock: in std_logic;
        reset: in std_logic;
        message: in std_logic_vector( LCDdatasize - 1 downto 0 );
        enable: in std_logic; 
        lcd_data: out std_logic_vector( LCDdatasize - 1 downto 0 );
        lcd_rs: out std_logic;
        lcd_rw: out std_logic;
        lcd_enable: out std_logic;
		LED1_out: out std_logic_vector(LEDseg - 1 downto 0);
		LED2_out: out std_logic_vector(LEDseg - 1 downto 0);
	    decimal_out: out std_logic);
		
end entity LCD;

architecture structural of LCD is
signal enable_wire: std_logic;
signal reset_wire: std_logic;
signal tmp: std_logic;
signal slowclock_wire:std_logic;

begin

tmp <= '1';
decimal_out <= '0';

clock_divider: clk_divider generic map (CLOCK_DIV => 33)  -- divide to 1 us clock
    						port map (clk_in => clock,
									 reset => tmp,
									 clk_out => slowclock_wire);

enable_debouncer: debouncer generic map (CLOCK_DIV => 20)
							port map(clock => slowclock_wire, 
									 reset => tmp, 
									 signal_in => enable, 
									 signal_out =>enable_wire);


reset_debouncer: debouncer generic map (CLOCK_DIV => 20)
						   port map (clock => slowclock_wire,
									 reset => tmp, 
									 signal_in => reset, 
									 signal_out => reset_wire);


LCDcontroller: LCDdriver generic map(big_delay =>  2500,
									small_delay => 500,
									setup_delay => 50)
						port map 

									(clock => slowclock_wire,
									reset => reset_wire,
									message => message,
									enable => enable_wire,
									lcd_data => lcd_data,
									lcd_rs => lcd_rs,
									lcd_rw => lcd_rw,
									lcd_enable => lcd_enable,
									LED1=>LED1_out,
									LED2=>LED2_out);

end architecture;
